var React = require('react');

var Translator = function () {
    this.warning = false;
    this.messages = [];
    this._lang = null;
    this.store = null;
    this.countReducers = {};
};

Translator.prototype.setCountReducer = function (lang, callback) {
    this.countReducers[lang] = callback;
}

Translator.prototype.registerMessage = function (message) {
    this.messages.push(message);
}

Translator.prototype.reduceCount = function (count, lang) {
    if(this.countReducers[lang])
        return this.countReducers[lang](count);
    return count;
}


Translator.prototype.unregisterMessage = function (message) {
    var self = this;
    this.messages = this.messages.filter(function(msg) {
        return msg !== message;
    });
}

Translator.prototype.displayWarning = function (val) {
    this.warning = val;
}

Translator.prototype.setLang = function (lang) {
    this._lang = lang;
}

Translator.prototype.getLang = function () {
    return this._lang;
}

Translator.prototype._exec = function (key, dictionary, lang) {
    var parts = key.split('.');
    var ret = '';
    var currentVal = dictionary;
    for(var i in parts){
        if(typeof currentVal != 'object'){
            if(this.warning)
                console.log('Translator['+lang+']: missing translation for key: ' + key);
            return key;
        } else{
            currentVal = currentVal[parts[i]];
        }
    }
    if(typeof currentVal != 'string'){
        if(this.warning)
            console.log('Translator['+lang+']: missing translation for key: ' + key);
        return key;
    }
    return currentVal;
}

Translator.prototype.translate = function (message, count) {
    if(!message){
        if(this.warning)
            console.log('Can not translate empty message.');
        return '';
    }
    var state = this.store.getState();
    var dictionary = state.dictionaries[state.lang];
    if(count){
        count = this.reduceCount(count, state.lang);
        message += '.'+count;
    }
    var ret = this._exec(message, dictionary, state.lang);
    return ret;
}

Translator.prototype.setStore = function (store) {
    this.store = store;
    this.setLang(store.getState().lang);
    var self = this;
    store.subscribe(function() {
        var state = store.getState();
        if (state.lang != self.getLang()) {
            self.setLang(state.lang);
            self.messages.map(function(message) {
                message.setState({})
            })
        }
        self.lang = store.lang;
    });
}

var translator = new Translator();

var Message = React.createClass({
    componentDidMount: function () {
        translator.registerMessage(this);
    },

    componentWillUnmount: function(){
        translator.unregisterMessage(this);
    },

    format: function(str, arr){
        if(str && arr){
            return str.replace(/%(\d+)/g, function(_,m) {
                return arr[--m];
            });
        }
        return str;
    },

    render: function () {
        return React.createElement(
            this.props.element ? this.props.element : "span",
            null,
            this.format(translator.translate(this.props.msg ? this.props.msg : this.props.children, this.props.count), this.props.params)
        );
    }
});

translator.Message = Message;

module.exports = translator;